# Copyright (c) 2003-2004 Chalmers University of Technology
# Authors: Jean-Marc Orliaguet <jmo@ita.chalmers.se>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

__author__ = "Jean-Marc Orliaguet <jmo@ita.chalmers.se>"

""" CPSSkins utilities """

from Acquisition import  aq_base
from Products.CMFCore.utils import getToolByName
from zLOG import LOG, DEBUG, ERROR
from DateTime import DateTime
import random

def rebuild_properties(obj):
    """ This method rebuilds an object's property map (_properties) and  
        to keep the property values up to date with the current property
        definition.

        If an unknown property field is found the method
        will look for a dictionary item called 'default' and the 
        property value will be filled with the value of 'default'.
       
        Otherwise the property value will be reset to '' or 0 depending
        on its type.
    """

    #
    # Rebuild property maps
    #
    for prop_map in obj.propertyMap():
        for prop in obj._properties:
            if prop['id'] == prop_map['id']:
                for key in prop_map.keys():
                    prop[key] = prop_map.get(key, None)
                             
    #
    # Rebuild properties
    #
    prop_dict = {}       
    prop_ids = obj.propertyIds()
    for prop_id in prop_ids:       
        try:
            prop_value = obj.getProperty(prop_id)
        except AttributeError:
            prop_value = None

        if prop_value is None:
            #
            # Look for a default value ('default' in the _properties dictionary)
            #
            default_value = None
            for prop_map in obj.propertyMap():
                if prop_id == prop_map['id']:
                    default_value = prop_map.get('default', None)
                    break

            if default_value is not None:
                prop_value = default_value
            else:   
                prop_type = obj.getPropertyType(prop_id)
                if prop_type in ['int', 'boolean']:
                    prop_value = 0

                if prop_type in ['lines', 'string', 'text', \
                                 'selection', 'multiple selection']:
                    prop_value = ''
 
        if prop_value is not None and not hasattr(prop_value, 'aq_base'):
            prop_dict[prop_id] = prop_value

    obj.manage_delProperties(prop_ids, REQUEST=None)
    if prop_dict is not None:
        obj.manage_changeProperties(prop_dict)
    return prop_dict



def callAction(self, actionid, **kw):
    """
    Call the given action.
    """

    ti = self.getTypeInfo() 
    try: 
        action = ti.getActionById(actionid)
    except:
        return
    else:
        try:
            meth = self.restrictedTraverse(action)
        except:
            return 
        else:
            if getattr(aq_base(meth), 'isDocTemp', 0):
                return apply(meth, (self, self.REQUEST), kw)
            else:
                return apply(meth, (), kw)


def getFreeId(container=None, try_id=None):
    """ This method looks for a free object id in the current folder
        and returns an id.
    """
                  
    if container is None:
        return

    ids = container.objectIds()
    if try_id is not None:
        if not try_id in ids:
            return try_id
    while 1:
        randomid = str(random.randrange(1, 2147483600))
        if not randomid in ids:
            break
    return randomid


def getFreeTitle(container=None, title='Noname'):
    """ This method looks for a free object title in a container
        and returns a title.

        If a title is passed as a parameter, the method will check first  
        if the title is free.
      
        Otherwise a figure (1, 2, ...) will be added to the title 
        until a free title is found.
    """

    if container is None:
        return

    titles = [getattr(obj, 'title', None) for obj in container.objectValues()]

    new_title = title
    i = 0
    while 1:
        if titles is None: 
            break
        if new_title not in titles:
            break
        i = i + 1
        new_title = title + str(i)

    return new_title



def canonizeStyleTitle(title=''):
    """ This method canonizes a CSS class title so that it can be used
        in a CSS file.
    """

    newtitle = ''
    for i in range(len(title)):
        character = title[i]
        if character.isalnum():
            newtitle += character
 
    return newtitle

def canonizeId(self):
    """ This method canonizes a Zope Id 
    """

    current_id = self.getId()
    new_id = ''
    for i in range(len(current_id)):
        character = current_id[i]
        if character.isalnum() or character in ['_', '-']:
            new_id += character

    if new_id != current_id:
        container = self.aq_parent
        new_id = getFreeId(container)
        container.manage_renameObjects([current_id], [new_id])


def verifyThemePerms(self):
    """ This method sets the correct permissions on an object 
        belonging to a theme, and reindexes the object's security matrix.
    """

    for perm in ('View', 'Manage Themes'):
        self.manage_permission(perm, [], acquire=1)
    self.reindexObjectSecurity()
 

def detectPortalType(self):
    """ This methods attempts to detect the portal type
    """  
             
    portal_type = 'CMF'
    portal = self.portal_url.getPortalObject()
    meta_type = portal.meta_type
             
    # CPS2
    if getToolByName(self, 'portal_hierarchies', None) is not None:
        return 'CPS2'
             
    # CPS3
    if meta_type == 'CPSDefault Site':
        return 'CPS3'
            
    # Plone
    migrationtool = getToolByName(self, 'portal_migration', None)
    if migrationtool is not None:
        version = migrationtool.getInstanceVersion()
        if version.startswith('2'):
            return 'Plone2'
        if version.startswith('1'):
            return 'Plone'
          
    return portal_type


def getObjectVisibility(self, REQUEST):
    """ Return 1 if the object if visible """

    if self.languages:
        portal = getToolByName(self, 'portal_url').getPortalObject()
        if hasattr(portal, 'Localizer'):
            lctool = portal.Localizer
        else:
            lctool = getToolByName(self, 'portal_messages', None)
        if lctool is None:
            return 0
        selected_language = lctool.get_selected_language()
        lang_list = self.languages
        if selected_language not in lang_list:
            return 0

    visibility = self.visibility
    if visibility == 'always':
        return 1

    if visibility == 'time_limited':
        now = DateTime()
        effective_date = self.EffectiveDate()
        expiration_date = self.ExpirationDate()
        if effective_date != 'None':
           if now < DateTime(effective_date):
              return 0
        if expiration_date != 'None':
           if now >= DateTime(expiration_date):
              return 0
        return 1

    if visibility == 'if_authenticated':
        mtool = getToolByName(self, 'portal_membership')
        if not mtool.isAnonymousUser():
           return 1       
                            
    if visibility == 'if_anonymous':
        mtool = getToolByName(self, 'portal_membership')
        if mtool.isAnonymousUser():
            return 1       
                            
    if visibility in ['only_in', 'everywhere_except_in', \
                      'starting_from', 'up_till']:
        utool = getToolByName(self, 'portal_url')
        portal_url = utool(relative=0)
                             
        if REQUEST is None:
            return 0      
                            
        # simulated URL
        rurl = REQUEST.form.get('sim_url', None)
        if rurl:           
            url = rurl
                            
        # real URL         
        if rurl is None:
            url_obj = REQUEST.get('context_obj', None)
            if url_obj:     
                url = utool.getRelativeContentURL(url_obj)  
                if url:    
                    url = '/' + url + '/'
                else:      
                    url = '/'
                               
        if url is None:    
            return 0   
                        
        url_path = url
        paths = [(p == '/' and '' or p) for p in self.visibility_paths]
                            
        if visibility == 'only_in':
            if url_path in paths:
                return 1 
                               
        if visibility == 'everywhere_except_in':
            if url_path not in paths:
                return 1 
                             
        if visibility == 'starting_from':
            for p in paths: 
                if url_path.startswith(p):
                    return 1 
                               
        if visibility == 'up_till':
            for p in paths: 
                if url_path.startswith(p) and url_path != p:
                     return 0 
            return 1        
                            
    if visibility == 'if_secure_connection':
        if REQUEST is not None:
            theBase = REQUEST['BASE0'].split('//')[0]
            if theBase == 'https:': 
                return 1   


def getApplicableStylesFor(self):
    """ Returns the list of styles by meta type that are 
        applicable to the current object
    """
    list = []
    tmtool = getToolByName(self, 'portal_themes')

    styles = [s.Metatype() for s in tmtool.listStyleTypes()]
    for propid in self.propertyIds():
        prop_map = self.propertyMap()
        for obj in prop_map:            
            if obj['id'] == propid:                
                meta_type = obj.get('meta_type')
                if meta_type and meta_type in styles:
                    list.append(meta_type)
    return list

